@@ -9,7 +9,7 @@
 #include <rz_egg.h>
 #include <rz_bp.h>
 #include <rz_io.h>
-#include <rz_msg_digest.h>
+#include <rz_hash.h>
 #include <rz_syscall.h>
 #include <rz_cmd.h>
 
@@ -51,15 +51,8 @@ RZ_LIB_VERSION_HEADER(rz_debug);
 #define PTRACE_SYSCALL    PT_STEP
 #endif
 
-#define CMD_CHECK_DEBUG_DEAD(core) \
-	do { \
-		if (rz_debug_is_dead(core->dbg)) { \
-			rz_cons_println("Debugging is not enabled. Run ood?"); \
-			return RZ_CMD_STATUS_ERROR; \
-		} \
-	} while (0)
 #define SNAP_PAGE_SIZE    4096
-#define CHECK_POINT_LIMIT 0x100000 //TODO: take the benchmark
+#define CHECK_POINT_LIMIT 0x100000 // TODO: take the benchmark
 /*
  * states that a process can be in
  */
@@ -190,14 +183,14 @@ typedef struct {
 typedef struct rz_debug_checkpoint_t {
 	int cnum;
 	RzRegArena *arena[RZ_REG_TYPE_LAST];
-	RzList *snaps; // <RzDebugSnap>
+	RzList /*<RzDebugSnap *>*/ *snaps;
 } RzDebugCheckpoint;
 
 typedef struct rz_debug_session_t {
 	ut32 cnum;
 	ut32 maxcnum;
 	RzDebugCheckpoint *cur_chkpt;
-	RzVector *checkpoints; /* RzVector<RzDebugCheckpoint> */
+	RzVector /*<RzDebugCheckpoint>*/ *checkpoints;
 	HtUP *memory; /* RzVector<RzDebugChangeMem> */
 	HtUP *registers; /* RzVector<RzDebugChangeReg> */
 	int reasontype /*RzDebugReasonType*/;
@@ -224,10 +217,10 @@ typedef struct rz_snap_entry {
 } RSnapEntry;
 
 typedef struct rz_debug_trace_t {
-	RzList *traces;
+	RzList /*<RzDebugTracepoint *>*/ *traces;
 	int count;
 	int enabled;
-	//int changed;
+	// int changed;
 	int tag;
 	int dup;
 	char *addresses;
@@ -247,7 +240,7 @@ typedef struct rz_debug_tracepoint_t {
 
 typedef struct rz_debug_t {
 	char *arch;
-	int bits; /// XXX: MUST SET ///
+	RZ_DEPRECATE int bits; ///< bad indicator for the bitness of the debuggee
 	int hitinfo;
 
 	int main_pid;
@@ -255,14 +248,13 @@ typedef struct rz_debug_t {
 	int tid; /* selected thread id */
 	int forked_pid; /* last pid created by fork */
 	int n_threads;
-	RzList *threads; /* NOTE: list contents are platform-specific */
+	RzList /*<void *>*/ *threads; /* NOTE: list contents are platform-specific */
 
 	char *malloc; /*choose malloc parser: 0 = glibc, 1 = jemalloc*/
 
 	/* dbg.* config options (see e?dbg)
 	 * NOTE: some settings are checked inline instead of tracked here.
 	 */
-	int bpsize; /* size of a breakpoint */
 	char *btalgo; /* select backtrace algorithm */
 	int btdepth; /* backtrace depth */
 	int regcols; /* display columns */
@@ -289,11 +281,8 @@ typedef struct rz_debug_t {
 	RzDebugTrace *trace;
 	HtUP *tracenodes;
 	RTree *tree;
-	RzList *call_frames;
-
+	RzList /*<RzDebugFrame *>*/ *call_frames;
 	RzReg *reg;
-	RzList *q_regs;
-	const char *creg; // current register value
 	RzBreakpoint *bp;
 	char *snap_path;
 
@@ -303,16 +292,17 @@ typedef struct rz_debug_t {
 
 	struct rz_debug_plugin_t *cur;
 	void *plugin_data;
-	RzList *plugins;
+	RzList /*<RzDebugPlugin *>*/ *plugins;
 
 	bool pc_at_bp; /* after a breakpoint, is the pc at the bp? */
 	bool pc_at_bp_set; /* is the pc_at_bp variable set already? */
 
 	RzEvent *ev;
+	RzHash *hash;
 
 	RzAnalysis *analysis;
-	RzList *maps; // <RzDebugMap>
-	RzList *maps_user; // <RzDebugMap>
+	RzList /*<RzDebugMap *>*/ *maps;
+	RzList /*<RzDebugMap *>*/ *maps_user;
 
 	bool trace_continue;
 	RzAnalysisOp *cur_op;
@@ -327,6 +317,8 @@ typedef struct rz_debug_t {
 	bool verbose;
 	bool main_arena_resolved; /* is the main_arena resolved already? */
 	int glibc_version;
+
+	bool nt_x86_xstate_supported; ///< Track whether X86_FEATURE_XSAVE feature is supported on current kernel
 } RzDebug;
 
 typedef struct rz_debug_desc_plugin_t {
@@ -336,7 +328,7 @@ typedef struct rz_debug_desc_plugin_t {
 	int (*write)(int fd, ut64 addr, int len);
 	int (*seek)(int fd, ut64 addr);
 	int (*dup)(int fd, int newfd);
-	RzList *(*list)(int pid);
+	RzList /*<RzDebugDesc *>*/ *(*list)(int pid);
 } RzDebugDescPlugin;
 
 typedef struct rz_debug_info_t {
@@ -369,39 +361,36 @@ typedef struct rz_debug_plugin_t {
 	ut32 bits;
 	const char *arch;
 	int canstep;
-	int keepio;
 	bool (*init)(RzDebug *dbg, void **user);
 	void (*fini)(RzDebug *debug, void *user);
 	/* life */
 	RzDebugInfo *(*info)(RzDebug *dbg, const char *arg);
-	int (*startv)(int argc, char **argv);
 	int (*attach)(RzDebug *dbg, int pid);
 	int (*detach)(RzDebug *dbg, int pid);
 	int (*select)(RzDebug *dbg, int pid, int tid);
-	RzList *(*threads)(RzDebug *dbg, int pid);
-	RzList *(*pids)(RzDebug *dbg, int pid);
-	RzList *(*tids)(RzDebug *dbg, int pid);
-	RFList (*backtrace)(RzDebug *dbg, int count);
+	RzList /*<RzDebugPid *>*/ *(*threads)(RzDebug *dbg, int pid);
+	RzList /*<RzDebugPid *>*/ *(*pids)(RzDebug *dbg, int pid);
 	/* flow */
 	int (*stop)(RzDebug *dbg);
-	int (*step)(RzDebug *dbg);
-	int (*step_over)(RzDebug *dbg);
+	bool (*step)(RzDebug *dbg);
+	bool (*step_over)(RzDebug *dbg);
 	int (*cont)(RzDebug *dbg, int pid, int tid, int sig);
-	int (*wait)(RzDebug *dbg, int pid);
+	RzDebugReasonType (*wait)(RzDebug *dbg, int pid);
 	bool (*gcore)(RzDebug *dbg, char *path, RzBuffer *dest);
 	bool (*kill)(RzDebug *dbg, int pid, int tid, int sig);
-	RzList *(*kill_list)(RzDebug *dbg);
+	RzList /*<void *>*/ *(*kill_list)(RzDebug *dbg);
 	int (*contsc)(RzDebug *dbg, int pid, int sc);
-	RzList *(*frames)(RzDebug *dbg, ut64 at);
+	RzList /*<RzDebugFrame *>*/ *(*frames)(RzDebug *dbg, ut64 at);
 	RzBreakpointCallback breakpoint; /// Callback to be used for RzBreakpoint. When called, RzBreakpoint.user points to the RzDebug.
 	// XXX: specify, pid, tid, or RzDebug ?
 	int (*reg_read)(RzDebug *dbg, int type, ut8 *buf, int size);
-	int (*reg_write)(RzDebug *dbg, int type, const ut8 *buf, int size); //XXX struct rz_regset_t regs);
+	int (*reg_write)(RzDebug *dbg, int type, const ut8 *buf, int size); // XXX struct rz_regset_t regs);
+	bool (*sync_registers)(RzDebug *dbg, RzReg *reg, bool to_debugger);
 	char *(*reg_profile)(RzDebug *dbg);
 	int (*set_reg_profile)(RzDebug *dbg, const char *str);
 	/* memory */
-	RzList *(*map_get)(RzDebug *dbg);
-	RzList *(*modules_get)(RzDebug *dbg);
+	RzList /*<RzDebugMap *>*/ *(*map_get)(RzDebug *dbg);
+	RzList /*<RzDebugMap *>*/ *(*modules_get)(RzDebug *dbg);
 	RzDebugMap *(*map_alloc)(RzDebug *dbg, ut64 addr, int size, bool thp);
 	int (*map_dealloc)(RzDebug *dbg, ut64 addr, int size);
 	int (*map_protect)(RzDebug *dbg, ut64 addr, int size, int perms);
@@ -423,14 +412,28 @@ typedef struct rz_debug_pid_t {
 	ut64 pc;
 } RzDebugPid;
 
+typedef struct rz_backtrace_t {
+	RzDebugFrame *frame;
+	RzAnalysisFunction *fcn;
+	char *desc;
+	char *pcstr;
+	char *spstr;
+	char *flagdesc;
+	char *flagdesc2;
+} RzBacktrace;
+
+typedef struct rz_debug_esil_watchpoint_t {
+	int rwx;
+	int dev;
+	char *expr;
+} RzDebugEsilWatchpoint;
+
 #ifdef RZ_API
-RZ_API RzDebug *rz_debug_new(int hard);
+RZ_API RZ_OWN RzDebug *rz_debug_new(RZ_BORROW RZ_NONNULL RzBreakpointContext *bp_ctx);
 RZ_API RzDebug *rz_debug_free(RzDebug *dbg);
 
 RZ_API int rz_debug_attach(RzDebug *dbg, int pid);
 RZ_API int rz_debug_detach(RzDebug *dbg, int pid);
-RZ_API int rz_debug_startv(RzDebug *dbg, int argc, char **argv);
-RZ_API int rz_debug_start(RzDebug *dbg, const char *cmd);
 
 /* reason we stopped */
 RZ_API RzDebugReasonType rz_debug_stop_reason(RzDebug *dbg);
@@ -450,20 +453,17 @@ RZ_API int rz_debug_continue_syscall(RzDebug *dbg, int sc);
 RZ_API int rz_debug_continue_syscalls(RzDebug *dbg, int *sc, int n_sc);
 RZ_API int rz_debug_continue(RzDebug *dbg);
 RZ_API int rz_debug_continue_kill(RzDebug *dbg, int signal);
-#if __WINDOWS__
 RZ_API int rz_debug_continue_pass_exception(RzDebug *dbg);
-#endif
 
 /* process/thread handling */
 RZ_API bool rz_debug_select(RzDebug *dbg, int pid, int tid);
-//RZ_API int rz_debug_pid_add(RzDebug *dbg);
-//RZ_API int rz_debug_pid_add_thread(RzDebug *dbg);
-//RZ_API int rz_debug_pid_del(RzDebug *dbg);
-//RZ_API int rz_debug_pid_del_thread(RzDebug *dbg);
-RZ_API int rz_debug_pid_list(RzDebug *dbg, int pid, char fmt);
+// RZ_API int rz_debug_pid_add(RzDebug *dbg);
+// RZ_API int rz_debug_pid_add_thread(RzDebug *dbg);
+// RZ_API int rz_debug_pid_del(RzDebug *dbg);
+// RZ_API int rz_debug_pid_del_thread(RzDebug *dbg);
 RZ_API RzDebugPid *rz_debug_pid_new(const char *path, int pid, int uid, char status, ut64 pc);
 RZ_API RzDebugPid *rz_debug_pid_free(RzDebugPid *pid);
-RZ_API RzList *rz_debug_pids(RzDebug *dbg, int pid);
+RZ_API RzList /*<RzDebugPid *>*/ *rz_debug_pids(RzDebug *dbg, int pid);
 
 RZ_API bool rz_debug_set_arch(RzDebug *dbg, const char *arch, int bits);
 RZ_API bool rz_debug_use(RzDebug *dbg, const char *str);
@@ -477,33 +477,30 @@ RZ_API ut64 rz_debug_get_baddr(RzDebug *dbg, const char *file);
 RZ_API void rz_debug_signal_init(RzDebug *dbg);
 RZ_API int rz_debug_signal_send(RzDebug *dbg, int num);
 RZ_API int rz_debug_signal_what(RzDebug *dbg, int num);
-RZ_API int rz_debug_signal_resolve(RzDebug *dbg, const char *signame);
-RZ_API const char *rz_debug_signal_resolve_i(RzDebug *dbg, int signum);
 RZ_API void rz_debug_signal_setup(RzDebug *dbg, int num, int opt);
 RZ_API int rz_debug_signal_set(RzDebug *dbg, int num, ut64 addr);
-RZ_API void rz_debug_signal_list(RzDebug *dbg, RzOutputMode mode);
+RZ_API bool rz_debug_can_kill(RzDebug *dbg);
 RZ_API int rz_debug_kill(RzDebug *dbg, int pid, int tid, int sig);
-RZ_API RzList *rz_debug_kill_list(RzDebug *dbg);
+RZ_API RzList /*<void *>*/ *rz_debug_kill_list(RzDebug *dbg);
 // XXX: must be uint64 action
 RZ_API int rz_debug_kill_setup(RzDebug *dbg, int sig, int action);
 
 /* handle.c */
 RZ_API void rz_debug_plugin_init(RzDebug *dbg);
-RZ_API int rz_debug_plugin_set(RzDebug *dbg, const char *str);
-RZ_API bool rz_debug_plugin_add(RzDebug *dbg, RzDebugPlugin *foo);
+RZ_API bool rz_debug_plugin_add(RzDebug *dbg, RZ_NONNULL RzDebugPlugin *plugin);
+RZ_API bool rz_debug_plugin_del(RzDebug *dbg, RZ_NONNULL RzDebugPlugin *plugin);
 RZ_API bool rz_debug_plugin_set_reg_profile(RzDebug *dbg, const char *str);
 
 /* memory */
-RZ_API RzList *rz_debug_modules_list(RzDebug *);
+RZ_API RzList /*<RzDebugMap *>*/ *rz_debug_modules_list(RzDebug *);
 RZ_API RzDebugMap *rz_debug_map_alloc(RzDebug *dbg, ut64 addr, int size, bool thp);
 RZ_API int rz_debug_map_dealloc(RzDebug *dbg, RzDebugMap *map);
-RZ_API RzList *rz_debug_map_list_new(void);
+RZ_API RzList /*<RzDebugMap *>*/ *rz_debug_map_list_new(void);
 RZ_API RzDebugMap *rz_debug_map_get(RzDebug *dbg, ut64 addr);
-RZ_API RzDebugMap *rz_debug_map_new(char *name, ut64 addr, ut64 addr_end, int perm, int user);
-RZ_API void rz_debug_map_free(RzDebugMap *map);
-RZ_API void rz_debug_map_print(RzDebug *dbg, ut64 addr, RzCmdStateOutput *state);
+RZ_API RZ_OWN RzDebugMap *rz_debug_map_new(RZ_NULLABLE char *name, ut64 begin, ut64 end, int perm, int user);
+RZ_API void rz_debug_map_free(RZ_NULLABLE RzDebugMap *map);
 RZ_API void rz_debug_map_list_visual(RzDebug *dbg, ut64 addr, const char *input, int colors);
-RZ_API RzList *rz_debug_map_list(RzDebug *dbg, bool user_map);
+RZ_API RZ_BORROW RzList /*<RzDebugMap *>*/ *rz_debug_map_list(RzDebug *dbg, bool user_map);
 
 /* descriptors */
 RZ_API RzDebugDesc *rz_debug_desc_new(int fd, char *path, int perm, int type, int off);
@@ -521,7 +518,6 @@ RZ_API bool rz_debug_reg_profile_sync(RzDebug *dbg);
 RZ_API int rz_debug_reg_sync(RzDebug *dbg, int type, int write);
 RZ_API int rz_debug_reg_set(RzDebug *dbg, const char *name, ut64 num);
 RZ_API ut64 rz_debug_reg_get(RzDebug *dbg, const char *name);
-RZ_API ut64 rz_debug_reg_get_err(RzDebug *dbg, const char *name, int *err, utX *value);
 
 RZ_API ut64 rz_debug_execute(RzDebug *dbg, const ut8 *buf, int len, int restore);
 RZ_API bool rz_debug_map_sync(RzDebug *dbg);
@@ -529,21 +525,17 @@ RZ_API bool rz_debug_map_sync(RzDebug *dbg);
 RZ_API int rz_debug_stop(RzDebug *dbg);
 
 /* backtrace */
-RZ_API RzList *rz_debug_frames(RzDebug *dbg, ut64 at);
+RZ_API RzList /*<RzDebugFrame *>*/ *rz_debug_frames(RzDebug *dbg, ut64 at);
 
 RZ_API bool rz_debug_is_dead(RzDebug *dbg);
 RZ_API int rz_debug_map_protect(RzDebug *dbg, ut64 addr, int size, int perms);
-RZ_API ut64 rz_debug_arg_get(RzDebug *dbg, const char *cc, int num);
-RZ_API bool rz_debug_arg_set(RzDebug *dbg, const char *cc, int num, ut64 value);
 
 /* breakpoints (most in rz_bp, this calls those) */
-RZ_API RzBreakpointItem *rz_debug_bp_add(RzDebug *dbg, ut64 addr, int hw, bool watch, int rw, char *module, st64 m_delta);
+RZ_API RZ_BORROW RzBreakpointItem *rz_debug_bp_add(RZ_NONNULL RzDebug *dbg, ut64 addr, int hw, bool watch, int rw, RZ_NULLABLE const char *module, st64 m_delta);
 RZ_API void rz_debug_bp_rebase(RzDebug *dbg, ut64 old_base, ut64 new_base);
 RZ_API void rz_debug_bp_update(RzDebug *dbg);
 
 /* pid */
-RZ_API int rz_debug_thread_list(RzDebug *dbg, int pid, char fmt);
-
 RZ_API void rz_debug_tracenodes_reset(RzDebug *dbg);
 
 RZ_API void rz_debug_trace_reset(RzDebug *dbg);
@@ -551,7 +543,9 @@ RZ_API int rz_debug_trace_pc(RzDebug *dbg, ut64 pc);
 RZ_API void rz_debug_trace_op(RzDebug *dbg, RzAnalysisOp *op);
 RZ_API void rz_debug_trace_at(RzDebug *dbg, const char *str);
 RZ_API RzDebugTracepoint *rz_debug_trace_get(RzDebug *dbg, ut64 addr);
-RZ_API void rz_debug_trace_list(RzDebug *dbg, int mode, ut64 offset);
+RZ_API void rz_debug_trace_print(RzDebug *dbg, RzCmdStateOutput *state, ut64 offset);
+RZ_API RZ_OWN RzList /*<RzListInfo *>*/ *rz_debug_traces_info(RzDebug *dbg, ut64 offset);
+RZ_API void rz_debug_traces_ascii(RzDebug *dbg, ut64 offset);
 RZ_API RzDebugTracepoint *rz_debug_trace_add(RzDebug *dbg, ut64 addr, int size);
 RZ_API RzDebugTrace *rz_debug_trace_new(void);
 RZ_API void rz_debug_trace_free(RzDebugTrace *dbg);
@@ -570,7 +564,7 @@ RZ_API ut64 rz_debug_esil_step(RzDebug *dbg, ut32 count);
 RZ_API ut64 rz_debug_esil_continue(RzDebug *dbg);
 RZ_API void rz_debug_esil_watch(RzDebug *dbg, int rwx, int dev, const char *expr);
 RZ_API void rz_debug_esil_watch_reset(RzDebug *dbg);
-RZ_API void rz_debug_esil_watch_list(RzDebug *dbg);
+RZ_API RZ_BORROW RzList /*<RzDebugEsilWatchpoint *>*/ *rz_debug_esil_watch_list(RzDebug *dbg);
 RZ_API int rz_debug_esil_watch_empty(RzDebug *dbg);
 RZ_API void rz_debug_esil_prestep(RzDebug *d, int p);
 
@@ -586,15 +580,15 @@ RZ_API void rz_debug_session_deserialize(RzDebugSession *session, Sdb *db);
 RZ_API bool rz_debug_session_save(RzDebugSession *session, const char *file);
 RZ_API bool rz_debug_session_load(RzDebug *dbg, const char *file);
 RZ_API bool rz_debug_trace_ins_before(RzDebug *dbg);
-RZ_API bool rz_debug_trace_ins_after(RzDebug *dbg);
+RZ_API bool rz_debug_trace_ins_after(RZ_NONNULL RzDebug *dbg);
 
 RZ_API RzDebugSession *rz_debug_session_new(void);
 RZ_API void rz_debug_session_free(RzDebugSession *session);
 
 RZ_API RzDebugSnap *rz_debug_snap_map(RzDebug *dbg, RzDebugMap *map);
 RZ_API bool rz_debug_snap_contains(RzDebugSnap *snap, ut64 addr);
-RZ_API ut8 *rz_debug_snap_get_hash(RzDebugSnap *snap, RzMsgDigestSize *size);
-RZ_API bool rz_debug_snap_is_equal(RzDebugSnap *a, RzDebugSnap *b);
+RZ_API ut8 *rz_debug_snap_get_hash(RzDebug *dbg, RzDebugSnap *snap, RzHashSize *size);
+RZ_API bool rz_debug_snap_is_equal(RzDebug *dbg, RzDebugSnap *a, RzDebugSnap *b);
 RZ_API void rz_debug_snap_free(RzDebugSnap *snap);
 
 RZ_API int rz_debug_step_back(RzDebug *dbg, int steps);
@@ -602,6 +596,10 @@ RZ_API bool rz_debug_goto_cnum(RzDebug *dbg, ut32 cnum);
 RZ_API int rz_debug_step_cnum(RzDebug *dbg, int steps);
 RZ_API bool rz_debug_continue_back(RzDebug *dbg);
 
+/* serialize */
+RZ_API void rz_serialize_debug_save(RZ_NONNULL Sdb *db, RZ_NONNULL RzDebug *dbg);
+RZ_API bool rz_serialize_debug_load(RZ_NONNULL Sdb *db, RZ_NONNULL RzDebug *dbg, RZ_NULLABLE RzSerializeResultInfo *res);
+
 /* ptrace */
 #if HAVE_PTRACE
 static inline long rz_debug_ptrace(RzDebug *dbg, rz_ptrace_request_t request, pid_t pid, void *addr, rz_ptrace_data_t data) {
@@ -616,6 +614,7 @@ static inline void *rz_debug_ptrace_func(RzDebug *dbg, void *(*func)(void *), vo
 /* plugin pointers */
 extern RzDebugPlugin rz_debug_plugin_native;
 extern RzDebugPlugin rz_debug_plugin_esil;
+extern RzDebugPlugin rz_debug_plugin_dmp;
 extern RzDebugPlugin rz_debug_plugin_rap;
 extern RzDebugPlugin rz_debug_plugin_gdb;
 extern RzDebugPlugin rz_debug_plugin_bf;
@@ -634,10 +633,10 @@ extern RzDebugPlugin rz_debug_plugin_null;
 #endif
 
 /* regset */
-//RZ_API struct rz_regset_t* rz_regset_diff(struct rz_regset_t *a, struct rz_regset_t *b);
-//RZ_API int rz_regset_set(struct rz_regset_t *r, int idx, const char *name, ut64 value);
-//RZ_API struct rz_regset_t *rz_regset_new(int size);
-//RZ_API void rz_regset_free(struct rz_regset_t *r);
+// RZ_API struct rz_regset_t* rz_regset_diff(struct rz_regset_t *a, struct rz_regset_t *b);
+// RZ_API int rz_regset_set(struct rz_regset_t *r, int idx, const char *name, ut64 value);
+// RZ_API struct rz_regset_t *rz_regset_new(int size);
+// RZ_API void rz_regset_free(struct rz_regset_t *r);
 
 #if 0
 Missing callbacks
